class LoopNode :
	
	def __init__(self,) :
		self.syntax = []
		self.loop_body = []
	

def HasIndent(line,indent) :
	count = 0
	for l in line :
		if l == ' ' :
			count += 1
			continue
		break
	if count == indent :
		return True
	return False

def BuildTree(lines,indent,prev_node=None) :
	syntax = lines[0]
	node = LoopNode()
	node.syntax.append(syntax)
	if prev_node :
		prev_node.loop_body.append(node)
	i = 1
	N = len(lines)
	while i < N :
		line = lines[i]
		if line.split() == [] :
			i = i + 1
			continue
		if not HasIndent(line,indent) :
			print("indentation error :- {} ".format(indent),line)
			break
		if line.split()[0] == 'for' or line.split()[0] == 'while' :
			pos,_ = BuildTree(lines[i:],indent+4,prev_node=node)
			i = i + pos
			continue
		node.loop_body.append(line)
		i = i + 1
	return i,node
	
def PrintTree(node) :
	syntax = node.syntax[0].split()
	a = int(syntax[3])
	b = int(syntax[5])
	for i in range(a,b+1) :
		for line in node.loop_body :
			if type(line) == type(node) :
				PrintTree(line)
				continue
			print(line)
	return

"""
Loop = ['for i = 1 to 10 :','    h = h + 1','    for j = 1 to 10 :','        pass','        loop','    a = a + 1','    h = h + 1']
L1 = ['for i = 2 to 7',"    for j = 1 to 3","        for k = 1 to 3","            pass","    pass"]
L2 = ['for i = 1 to 3','    for j = 1 to 3','        nes_loop','        for k = 1 to 2','            nes_nes_loop','    pass']
_,node = BuildTree(Loop,4)
PrintTree(node)
print(node.loop_body)
_,node1 = BuildTree(L2,4)
PrintTree(node1)
print(node1.loop_body)
"""


class LoopProcessingUnit :
	
	def __init__(self,indent=4) :
		self.indent = indent
		self.loop_bodies = {}

	def FetchLoopBody(self,lines,indent) :
		pass
		
	def ProcessLoop(self,lines) :
		pass
		
	def EvaluateLoop(self,loop_bodies) :
		pass
	



def condition_split(line,op='+') :
	#1+arr[j]*arr[j+1]
	element = ''
	open_brackets = ['[','{','(']
	closing_brackets = [']','}',')']
	operators = ['+','-','*','/','^']
	elements = []
	N = len(line)
	found_bracket = False
	open_bracket = 0
	closing_bracket = 0
	for i in range(N) :
		
		if line[i] in open_brackets :
			element += line[i]
			found_bracket = True
			open_bracket += 1
			continue
		if line[i] in closing_brackets :
			element += line[i]
			closing_bracket += 1
			if closing_bracket == open_bracket :
				found_bracket = False
				closing_bracket = 0
				open_bracket = 0
			if i+1 == N :
				elements.append(element)
			continue
		if found_bracket :
			element += line[i]
			continue
		if op == 'any' :
			pos = -1
			if line[i] in operators :
				if i-1 == -1 :
					element += line[i]
					pos = i
				elif line[i-1] in operators :
					element += line[i]
					pos = i
			if  not line[i] in operators :
				element += line[i]
				pos = 0
				if i+1 == N :
					elements.append(element)
			else :
				if pos != -1 :
					continue
				elements.append(element)
				element = ''
		else :
			pos = -1
			if line[i] in operators :
				if i-1 == -1 :
					element += line[i]
					pos = 0
				#$elif line[i]
				elif line[i-1] in operators :
					if line[i] == '-' and line[i-1] == '-' :
						# - - = +
						pos = 0
						#element += '+'
					else :
						element += line[i]
						pos = 1
			if  line[i] != op :
				if pos == 0 and i-1 == -1 :
					continue
					
				element += line[i]
				if i+1 == N :
					elements.append(element)
				continue
			else :
				if pos != -1 :
					continue
				elements.append(element)
				element = ""
	return elements
	
"""
line = 'arr[i*1]*2*arr[j*2*2]*7'
line1 = '((1+2)+2)+5+6'
print(condition_split(line,op='*'))
print(condition_split(line1,op='+'))
print(condition_split('arr[i+1]',op='+'))
print(condition_split('2+3','+'))
print(condition_split('arr[i*1]+arr[k+1]/2+3','any'))
print(condition_split('pow(2+3,4*8)*pow(23,4+4)+f(2/3)','any'))
line = '-res+234*-544+-343'
print(condition_split(line,'any'))
print(condition_split('-2*2-3+4','+'))
print(condition_split('4-3+-2/4','-'))
print(condition_split('-3-2*2','any'))
print(condition_split('2+2','any'))
"""
#print(condition_split('-6-+6*8','-'))


class LoopNode :
	
	def __init__(self,) :
		self.syntax = []
		self.loop_body = []
	

def HasIndent(line,indent) :
	count = 0
	for l in line :
		if l == ' ' :
			count += 1
			continue
		break
	if count == indent :
		return True
	return False

def BuildTree(lines,prev_node=None,indent=4) :
	syntax = lines[0]
	node = LoopNode()
	node.syntax.append(syntax)
	pos = 0
	rec = 0
	if prev_node :
		prev_node.loop_body.append(node)
	for i in range(1,len(lines)) :
		if pos+i == len(lines) :
			rec = pos + i
			break
		line = lines[i+pos]
		if not HasIndent(line,indent) :
			break
		if line.split()[0] == 'for' or line.split()[0] == 'while' :
			pos,_ = BuildTree(lines[i+pos:],prev_node=node,indent = indent+4)
			pos = pos + 1
			continue
		print(line)
		node.loop_body.append(line)
	return rec,node
	
def PrintTree(node) :
	syntax = node.syntax[0].split()
	a = int(syntax[3])
	b = int(syntax[5])
	for i in range(a,b+1) :
		for line in node.loop_body :
			if type(line) == type(node) :
				PrintTree(line)
				continue
			print(line)
	return

"""
Loop = ['for i = 1 to 10 :','    h = h + 1','    for j = 1 to 10 :','        pass','        loop','    a = a + 1']
_,node = BuildTree(Loop)
PrintTree(node)
print(node.loop_body)
"""



class Compiler :
	
	def __init__(self,indent=4) :
		self.indent = indent
		self.error_code = False
		self.alpha = 'abcdefghijklmnopqrstuvwxyz'
		self.nums = '0123456789'
		self.GlobalVariables = {}
		self.LocalVariables = {}
		self.Variables = {'Global Variables' : self.GlobalVariables,'Local Variables' : self.LocalVariables}
		self.DataTypes = ['chars','int','float']
		self.LoopTypes = ['for','while']
		self.WhileLoopOp = ['<','>','<=','>=']
		self.chars_literals = self.alpha + self.nums
		self.int_literals = self.nums
		self.float_literals = self.nums
		self.Variables_Literals = {'int' : self.int_literals,'float' : self.float_literals,'chars' : self.chars_literals}
		
		
	
	def HasIndent(self,line,bias_indent=0) :
		if type(line) == type('a') :
			if '\n' in line :
				print("Error :- agr to this function HasIndent(str) must be of single line")
				return self.error_code
			count = 0
			for l in line :
				if l == ' ' :
					count = count  + 1
					continue
				break
			if count == self.indent + bias_indent :
				return True
			return False
		print("Error :- argument of incorrect data type provided to this function HasIndent(arg)")
		return
	
	def Is_single_line(self,line) :
		if type(line) == type('a') :
			if '\n' in line :
				return False
			return True
	
	def Split(self,st,char=' ') :
		new_str = []
		str_ = ''
		for s in st :
			if s == char :
				str_ = ''
				new_str.append(str_)
				continue
			str_ += s
		return new_str
		
	def ProcessVariable(self,line,scope='Global Variables') :
		if self.Is_single_line(line) :
			#DataType VarName = Value
			line = line.split()
			N = len(line)
			if N == 0 or N == 1 :
				return
			if '=' in line :
				if line[0] in self.DataTypes :
					N = 4
				else :
					N = 3
			if N == 4 :
				if line[0] in self.DataTypes :
					data_type = line[0]
					var_name = line[1]
					if not line[2] == '=' :
						print("Incorrect Syntax {}".format(line))
						return
					if not self.CheckVariableName(var_name) :
						 return
					var_val = line[3]
					var_lit = self.Variables_Literals[data_type]
					
					if data_type == 'int' :
						#int a = 2
						for v in var_val :
							if not v in var_lit :
								print("Incorrect Value assigned to the variable {}".format(line))
								return
						Var = self.Variables[scope]
						if var_name in Var.keys() :
							print("Error Variable is already defined {}".format(self.Variables[var_name]))
							return
						self.Variables[scope][var_name] = [var_val,data_type]
						return
					if data_type == 'float' :
						#float b = 1.2
						for v in range(len(var_val)) :
							if var_val[v] == '.' :
								continue
							if not var_val[v] in var_lit :
								print("Incorrect Value Assigned to the variable {}".format(line))
								return
						Var = self.Variables[scope]
						if var_name in Var.keys() :
							print("Error Variable is already defined {}".format(Var[var_name]))
							return
						self.Variables[scope][var_name] = [var_val,data_type]
						return
					if data_type == 'chars' :
						#chars name = '123'
						temp_val = ''
						if len(line[3:]) > 1 :
							N = len(line[3:])
							temp_line = line[3:]
							for v in range(N) :
								temp_val += temp_line[v]
								if v+1 == N :
									continue
								temp_val += ' '
							var_val = temp_val.lower()
						del temp_val
						if var_val[0] == "'" and var_val[-1] == "'" :
							for v in range(1,len(var_val)-1) :
								if var_val[v] == ' ' :
									continue
								if not var_val[v] in var_lit :
									print("Incorrect value assigned to the variable {}".format(line))
									return
							print(var_val)
							Var = self.Variables[scope]
							if var_name in Var.keys() :
								print("Error value of this variable is already defined {}".format(Var[var_name]))
								return
							self.Variables[scope][var_name] = [var_val,data_type]
							return
						print("Error string values must be put inside 'val' single quotation {}".format(var_val))
						return
			if N == 3  :
				# a = 1
				var_name = line[0]
				Var = self.Variables[scope]
				if not var_name in Var.keys() :
					print("Error Variable must be define before assigning values to it {}".format(line))
					return
				if line[1] != '=' :
					print("Error incorrect Syntax {}".format(line))
					return
				_,data_type = Var[var_name]
				var_lit = self.Variables_Literals[data_type]
				var_val = line[2]
				if data_type == 'int' :
						#int a = 2
						for v in var_val :
							if not v in var_lit :
								print("Incorrect Value assigned to the variable {}".format(line))
								return
						self.Variables[scope][var_name] = [var_val,data_type]
						return
				if data_type == 'float' :
					#float b = 1.2
						for v in range(len(var_val)) :
							if var_val[v] == '.' :
								continue
							if not var_val[v] in var_lit :
								print("Incorrect Value Assigned to the variable {}".format(line))
								return False
						self.Variables[scope][var_name] = [var_val,data_type]
						return
				if data_type == 'chars' :
						#chars name = 'Connie Talbot'
						#chars name = 'Mackenzie Foy'
						temp_val = ''
						temp_list = line[2:]
						if len(temp_list) > 1 :
							for i in range(len(temp_list)) :
								temp_val += temp_list[i]
								if i+1 == len(temp_list) :
									break
								temp_val += ' '
							var_val = temp_val
							del temp_val
						del temp_list
						if var_val[0] == "'" and var_val[-1] == "'" :
							for v in range(1,len(var_val)-1) :
								if var_val[v] == ' ' :
									continue
								if not var_val[v].lower() in var_lit :
									print("Incorrect value assigned to the variable {}".format(line))
									return
							self.Variables[scope][var_name] = [var_val,data_type]
							return
						print("Error string values must be put inside 'val' single quotation {}".format(var_val))
						return
	
	def ProcessLines(self,lines) :
		if type(lines) == type('a') :
			if '\n' in lines :
				lines = lines.split('\n')
		if lines == [] :
			return
		self.loop_bodies = {}
		i = 0
		loop_count = 0
		N = len(lines)
		while i < N :
			line = lines[i]
			if line.split() == [] :
				i = i + 1
				continue
			"""
			if not self.HasIndent(line,-self.indent) :
				print("Indentation error at line {} {}".format(i,line))
				return
			"""
			if line.split()[0] in self.LoopTypes :
				if not self.LoopSyntax(line) :
					return
				loop_body,pos = self.FetchLoopBody(lines[i+1:])
				if not loop_body:
					i = i + pos + 1
					continue
				self.loop_bodies['for_'+str(loop_count)] = loop_body
				pos = pos + 1
				i = i + 1
				continue
			print(line)
			self.ProcessVariable(line)
			i = i + 1
		
	def LoopSyntax(self,line) :
		line = line.split()
		#for int i = a to b :
		#while int i [<,>,<=,>=] b :
		if len(line) == 8 or len(line) == 7:
			#[for,int, i , =,a,to,  b,  :  ]
			#  0    1   2  3 4  5   6 7
			pos = 0
			if len(line) == 8 :
				pos = 1
			var_i = line[1+pos]
			if line[2+pos] != '=' :
				print("Syntax Error {}".format(line))
				return
			if line[4+pos] != 'to' :
				print("Syntax Error {}".format(line))
				return
			if line[6+pos] != ':' :
				print("Syntax Error {}".format(line))
				return
			a = line[3+pos]
			b = line[5+pos]
			vars = [var_i,a,b]
			for v in vars :
				if not self.CheckVariableName(v) :
					return
			return True	
		if len(line) == 6 or len(line) == 5 :
			#while int a < b :
			pos = 0
			if len(line) == 6 :
				pos = 1
			var_i = line[1+pos]
			if not line[2+pos] in self.WhileLoopOp :
				print("Error Syntax {}".format(line))
				return
			if not line[4+pos] == ':' :
				print("Syntax Error {}".format(line))
				return
			a =line[1+pos]
			b = line[3+pos]
			vars = [var_i,a,b]
			for v in vars :
				if not self.CheckVariableName(v) :
					return
			return True
			
	def FetchLoopBody(self,lines,indent=0) :
		loop_body = []
		i = 0
		N = len(lines)
		while i < N :
			line = lines[i]
			if line.split() == [] :
				i = i + 1
				continue
			if not self.HasIndent(line,indent) :
				break
			if line.split()[0] in self.LoopTypes :
				if self.LoopSyntax(line) :
					nes_loop,pos = self.FetchLoopBody(lines[i+1:],indent+self.indent)
					print("here nes ",nes_loop)
					pos = pos + 1
					loop_body.append(line)
					loop_body.append(nes_loop)
					i = i + pos
					continue
				print("Syntax Error {}".format(line))
				return [],0
			
			print("here ",line)
			loop_body.append(line)
			i = i + 1
			continue
		return loop_body,i
		
	def CheckVariableName(self,var_name) :
		if type(var_name) == type('a') :
			if '\n' in var_name :
				print("Error argument must be of single line while giving it to this function CheckVariableName(line)")
				return
			if var_name[0].lower() in self.alpha :
				for i in range(1,len(var_name)) :
					if not var_name[i].lower() in self.nums+self.alpha :
						print("Incorrect variable name {}".format(var_name))
						return
				if var_name in self.DataTypes :
					print("Incorrect Variable name {}".format(var_name))
				return True
			if var_name[0] in self.nums :
				for i in range(1,len(var_name)) :
					if var_name[i].lower() in self.alpha :
						print("Incorrect variable name {}".format(var_name))
						return
				return True

a = """
int a = 1
chars b = 'a'
float s = 2.0
s = 43.554387483
for int i = 1 to 2 :
     a = a + 1
     break
     for j = 1 to 4 :
     	pass
     	hey
     a = a + 1
chars f = 'fx'
f = 'gx'
chars name = 'Pinku Khare'
name = 'Prakamya Khare'
"""

c = Compiler(5)
c.ProcessLines(a)
print(c.GlobalVariables)
print(c.loop_bodies)	


def tokenize_defining_variable(line,self=None) :
	data_type = ""
	name = ""
	value  = ""
	found_eq = False
	for char in line :
		if not data_type in ['int','chars','float'] :
			if char == " " :
				continue
			data_type += char
			continue
		if char == '=' :
			found_eq = True
			continue
		if found_eq :
			value += char
			continue
		if char != ' ' :
			name += char
	return data_type,name,value

"""
line = 'int a=23723 '
print(tokenize_defining_variable(line))
line = 'chars name=  " Prakamya Khare sks " '
print(tokenize_defining_variable(line))
"""


from .ExpressionTree1 import ExpressionTree
from .StringAlgorithm.ConditionSplit import condition_split

def get_char_upto_n(compiler,line) :
	N  = len(line)
	k = 0
	chars  = ''
	for i in range(N) :
		if line[i] in compiler.int_literals+'-*/+' :
			k = i
			break
		chars += line[i]
	return chars,k

def process_functions(compiler,line,scope='global',scope_name='',exe=False) :
	
	if type(line) == type('a') :
		line = line.split()
	
	if len(line) == 1 and exe:
		#f(x)
		line = line[0]
		if not compiler.is_function(line) :
			return
		
		name,params_value,_ = compiler.get_func_param(line)
		
		if not name in compiler.func_body.keys() :
			return
		
		params_value = params_value.split(',')
		params_keys = compiler.func_body[name]['params']
		params = {}
		k = 0

		for i in range(len(params_value)) :
			val  = None
			if compiler.is_number(params_value[i]) :
				 val = params_value[i]
			elif compiler.check_variable_name(params_value[i]) :
				value = compiler.fetch_var_value(params_value[i],scope=scope,scope_name=scope_name)
				val = value
			params[params_keys[i]] = val
		
		expression  = compiler.func_body[name]['expression']
		N = len(expression)
		numerised_expression = ''
		
		if expression == "" :
			return
		
		for i in range(N) :
			if i+k >= N :
				break
				
			if expression[i+k] in compiler.alpha :
				chars,k = get_char_upto_n(compiler,expression[i+k:])
				k = k + 1
				if chars in params.keys() :
					numerised_expression += params[chars]
			numerised_expression += expression[i+k]		
		
		print(numerised_expression)
		value = compiler.compute_index(numerised_expression,scope=scope,scope_name=scope_name)
		return value
	
	if len(line) == 4 :
		#func f(x,y) = 2*x+3*y
		
		if line[0] != 'func' :
			return
		
		name,params,_= compiler.get_func_param(line[1])
		compiler.func_body[name] = {}
		compiler.func_body[name]['expression'] = line[-1]
		params = params.split(',')
		compiler.func_body[name]['params']  = params
		return

	if len(line) == 3 :
		#y = f(x)
		#y = f(x)/g(x)
		var_y = line[0]
		is_array = False
		index = None
		
		if '=' != line[1] :
			return
		
		if compiler.get_arr_index(var_y) :
			name,index,_ = compiler.get_arr_index(var_y)
			is_array = True
			var_y = name
			index = compiler.compute_index(index,scope=scope,scope_name=scope_name)
		
		
		if not compiler.fetch_variable(var_y,scope=scope,scope_name=scope_name) :
			print("error variable {} is not defined [process_function]".format(var_y))
			return
		
		value = line[2]
		
		if compiler.check_variable_name(value) :
			return
		
		has_op = False
		if len(condition_split(value,'any')) > 1 :
			has_op = True
		
		if has_op :
			value = compiler.compute_index(value,scope=scope,scope_name=scope_name,type='float')
		else :
			if not compiler.is_function(value) :
				return
			name,p,_ = compiler.get_func_param(value)
			if not name in compiler.func_body.keys() :
				return
			#that means it's single function
			value = process_functions(compiler,value,scope=scope,scope_name=scope_name,exe=True)
		
		#now assign the computed value to the variable y
		if is_array :
			compiler.assign_array(var_y,value,index=index,scope=scope,scope_name=scope_name)
			return
		compiler.assign_value(var_y,value,scope=scope,scope_name=scope_name)
		return	


from .ConditionSplitV2 import condition_split_v1
from .ConditionSplit import condition_split
from .ConditionSplitV3 import condition_split_v2
from .ExtractFuncNameParams import extract_name_param
from .TokenizeVariableDefinition import tokenize_defining_variable
from .SplitString import split


from .ExpressionTree1 import ExpressionTree
from .FunctionalProcessingUnit import FunctionProcessingUnit as FPU
from .LoopProcessingUnit2 import LoopProcessingUnit
from .ConditonalProcessingUnit import ConditionalProcessingUnit as CPU
from .StringAlgorithm.ConditionSplit import condition_split

class Compiler :
	
	def __init__(self,indent=4) :
		self.indent = indent
		self.loop_types = ['for','while']
		self.DataTypes = ['int','float','chars']
		self.ArrayDataTypes = ['int_arr','float_arr']
		self.int_literals = '0123456789'
		self.alpha = 'abcdefghijklmnopqrstuvwxyz'
		self.char_literals = 'abcdefghijklmnopqrstuvwxyz' + self.int_literals
		self.variable_error = "Error variable {} is not defined "
		self.value_error = "Incorrect value {}assigned to the variable {}"
		self.comment_syntax = '%'
		self.symbols = ["'",' ','.',">",'<','<=','=','/','?','*','-']
		self.func_body = {}
		self.functions = {}
		self.localVariables = {}
		self.func_scope = {}
		self.arithmetic_op = ['+','-','*','/']
		self.loop_scope = {}
		self.local_scope = {'func_scope' : self.func_scope,'loop_scope':self.loop_scope}
		self.localVariables = {'local_scope' : self.local_scope}
		self.globalVariables = {}
		self.Variables = {'global' : self.globalVariables,'local' : self.localVariables}
		self.literals = {'int' :self.int_literals,'float' : self.int_literals,'chars' : self.char_literals}
	
	def check_variable_name(self,name) :
		if name == '' :
			print("Error Name has not provided to the variable")
			return False
		if name[0] in self.alpha :
			for i in range(1,len(name)) :
				if not name[i].lower() in self.char_literals :
					print("Incorrect Variable name {}".format(name))
					return False
			return True
		print("Incorrect variable name {}".format(name))
		return False
	
	def has_indent(self,line,indent=0) :
		count = 0
		for l in line :
			if l == ' ' :
				count += 1
				continue
			break
		if count == self.indent+indent :
			return True
		return False
	
	def get_func_param(self,line) :
		if type(line) != type('a') :
			return 0,0,False
		name = ""
		param = ""
		is_found = False
		for i in line :
			if i == '(' :
				is_found = True
				continue
			if is_found :
				if i == ')' :
					break
				param += i
				continue
			name += i
		return name,param,is_found
	
	def fetch_var_value(self,name,scope='global',scope_name='') :
		is_var = False
		for n in name  :
			if n in self.alpha :
				is_var = True
				break
		if is_var :
			if scope == 'local' :
				var_scope = self.Variables[scope]['local_scope'][scope_name]
				if name in var_scope.keys() :
					val,_ = var_scope[name]
					return val
				if name in self.Variables[scope].keys() :
					val,_ = self.Variables[scope][name]
					return val
				print("Error variable {} is not defined".format(name))
				return
			if scope == 'global' :
				if name in self.Variables[scope].keys() :
					val,_ = self.Variables[scope][name]
					return val
				print("Error variable {} is not defined ".format(name))
				return
			return name
	
	def is_number(self,num) :
		is_num = True
		for n in num :
			if n == '.' :
				continue
			if not n in self.int_literals :
				is_num = False
				break
		return is_num
	
	def process_function(self,line,scope='global',scope_name='',exe=False,obj=None) :
		#func f(x) = 2*3+1
		line = line.split()
		st = "Error incorrect Syntax {}"
		if line == [] :
			return
		if len(line) == 1 :
			#f(x)
			name,param,_ = self.get_func_param(line[0])
			if name in self.func_body.keys() :
				#f(x,y)
				#f(res,2)
				tree,dic,param_f = self.func_body[name]
				params = param.split(',')
				for key,val in zip(param_f,params) :
					if self.is_number(val) :
						dic[key] = val
						continue
					value = self.fetch_var_value(val,scope=scope,scope_name=scope_name)
					dic[key] = value
				dic_ = dic
				tree.param_values = dic
				val = str(tree.computeTree(tree.head))
				tree.param_values = dic_
				return val
					
		if exe :
			#f(x) = g(x)*2
			#f(x,y) = x*g(y)/2
			#f(x) = x*g(res)/2
			#g(X)
			if not obj :
				return
			if type(line) == type('a') :
				line =line.split()
			if len(line) != 1 :
				print('Error function is not given alone {}'.format(line))
				return
			name,param,is_found = self.get_func_param(line[0])
			
			if not name in self.func_body.keys() :
				print("Error function {} is not defined".format(name))
				return
			tree_,g_dict,g_params = self.func_body[name]
			g_dict_ = g_dict
			has_found_all = True
			to_find = []
			for key in param :	
				if key in obj.param_values.keys() :
					g_dict[key] = obj.param_values[key]
					continue
				else :
					to_find.append(key)
					has_found_all = False
			
			if not has_found_all :
				for key in to_find :
					g_dict[key] = self.fetch_var_value(key,scope=scope,scope_name=scope_name)
					has_found_all = True
			tree_.param_values = g_dict
			output = str(tree_.computeTree(tree_.head))
			tree_.param_values = g_dict_
			return output
			
		if len(line) == 4 :
			if line[0] != 'func' :
				#print(st.format(line))
				return
			func_syntax = line[1]
			name = ""
			param = ""
			is_found = False
			name,param,is_found = self.get_func_param(line[1])
			
			if not is_found :
				print(st.format(line))
				return
			
			if not self.check_variable_name(name) :
				print("Error incorrect function name {}".format(name))
				return
			self.param_dict = {}
			for i in param :
				if i == ',' :
					continue
				if not i in self.alpha :
					print("Error parameter must be alphabets {}".format(param))
					return
			
			params = param.split(',')
			for p in params :
				self.param_dict[p] = ''
			expr = line[3]
			
			if line[2] != '=' :
				print(st.format(line))
				return
			
			has_op = False
			
			for op in self.arithmetic_op :
				if op in expr :
					has_op = True
			if has_op :
				tree = ExpressionTree(self.Variables,is_func=True,parameters=params,obj=self)
				tree.buildTree(expr)
				self.func_body[name] = [tree,self.param_dict,params]
				return
			self.func_body[name] = expr
			return
		if len(line) == 3 :
			#y = f(x)
			
			name,_,_ = self.get_func_param(line[2])
			if not name in self.func_body.keys() :
				return
			
			for op in self.arithmetic_op :
			    if op in line[2] :
			        return
			var_y = line[0]
			var_scope = None
			if scope == 'local' :
				is_found = False
				if var_y in self.Variables['local']['local_scope'][scope_name].keys() :
					var_scope = self.Variables['local']['local_scope'][scope_name]
					is_found = True
				if not is_found :
					if var_y in self.Variables['global'].keys() :
						var_scope = self.Variables['global']
						is_found = True
				if not is_found :
					print("Error variable {} is not defined".format(var_y))
					return	
			if scope == 'global' :
				if not var_y in self.Variables['global'].keys() :
					print("Error variable {} is not defined".format(var_y))
					return
				var_scope = self.Variables['global']
			name,param,is_found = self.get_func_param(line[2])
			if not is_found :
				#print(st.format(line))
				return
			if not name in self.func_body.keys() :
				print("Error function with name {} is not defined".format(name))
				return
			params = param.split(',')
			params_ = self.func_body[name][2]
			param_dict = self.func_body[name][1]
			for i,p in enumerate(params) :
				is_var = False
				for e in p :
					if e in self.alpha :
						is_var = True
						break
				if is_var :
					if scope == 'local' :
						if p in self.Variables['local']['local_scope'][scope_name].keys() :
							v,_ = self.Variables['local']['local_scope'][scope_name][p]
							param_dict[params_[i]] = v
							continue
						if p in self.Variables['global'].keys() :
							v,_= self.Variables['global'][p]
							param_dict[params_[i]] = v
							continue
						print("Error variable {} is not defined".format(p))
						return
					if scope == 'global' :
						if p in self.Variables['global'].keys() :
							v,_= self.Variables['global'][p]
							param_dict[params_[i]] = v
							continue
						print("Error variable {} is not defined".format(p))
						return
				param_dict[params_[i]] = params[i]
				
			tree = self.func_body[name][0]
			tree.param_values = param_dict
			value = str(tree.computeTree(tree.head))
			_,dt = var_scope[var_y]
			var_scope[var_y] = [value,dt]
			return True
			
	def process_Func(self,line,scope='global') :
			#y = pow(x)
			#where pow is a multiline function
			line = line.split()
			
			# y = 2*pow(x)
			if len(line) != 3 :
				return
			func_name = line[-1]
			name,param,_ = self.get_func_param(func_name)
			if name in self.functions.keys() :
				func = self.functions[name]
				val = func.executeTree(line)
				var_y = line[0]
				if scope == 'global' :
					if var_y in self.Variables['global'].keys() :
						_,dt = self.Variables['global'][var_y]
						self.Variables['global'][var_y] = [val,dt]
						return
				if var_y in self.Variables['local']['local_scope'][name].keys() :
					_,dt = self.Variables['local']['local_scope'][name][var_y]
					self.Variables['local']['local_scope'][name][var_y] = [val,dt]
					return
				if var_y in self.Variables['global'].keys() :
					_,dt = self.Variables['global'][var_y]
					self.Variables['global'][var_y] = [val,dt]
					return
			else :
				return
	
	def clean_line_from_op(self,line) :
		clean_line = ''
		for l in line :
			if l in self.arithmetic_op :
				clean_line += " "
				continue
			clean_line += l
		return clean_line
	
	def get_arr_index(self,arr) :
		name = ""
		index = ""
		count = 0
		is_arr = False
		found_brack = False
		N = len(arr)
		if arr[-1] != ']' :
			return False
		count = 1
		for i in range(N-1) :
			if arr[i] == '[' :
				count += 1
				found_brack = True
				continue
			if not found_brack :
				name += arr[i]
				continue
			index += arr[i]
		
		if count == 2 :
			is_arr = True
		if not self.check_variable_name(name) :
			is_arr = False
		
		return name,index,is_arr
		
	def have_two_brackets(self,var) :
		count = 0
		for i in var :
			if i == '[' or i == ']' :
				count += 1
		if count == 2 :
			return True
		return False
		
	def is_var_exist(self,var,scope='global',scope_name='') :
		if scope == 'global' :
			if var in self.Variables[scope].keys() :
				return True
			return False
		if var in self.Variables[scope]['local_scope'][scope_name].keys() :
			return True
		return self.is_var_exist(var,scope='global')
	
	def fetch_variable(self,var,scope='global',scope_name='') :
		if scope == 'global' :
			return self.Variables[scope][var]
		if var in self.Variables[scope]['local_scope'][scope_name].keys() :
			return self.Variables[scope]['local_scope'][scope_name][var]
		return self.fetch_variable(var,scope='global')
		
	def assign_array(self,name,value,index=None,scope='global',scope_name="") :
		if index :
			if scope == 'local' :
				if self.is_var_exist(name,scope=scope,scope_name=scope_name) :
					elements,dt = self.Variables['local']['local_scope'][scope_name][name]
					elements[index] = value
					self.Variables['local']['local_scope'][scope_name][name] = [elements,dt]
					return
			if self.is_var_exist(name,scope='global') :
				elements,dt = self.fetch_variable(name,scope='global')
				elements[index] = value
				self.Variables['global'][name] = [elements,dt]
				return
		return self.assign_value(name,value,scope=scope,scope_name=scope_name)
		
	def define_variable(self,variable,value,data_type,scope='global',scope_name="") :
		if scope == 'global' :
			self.Variables[scope][variable] = [value,data_type]
			return
		if scope == 'local' :
			self.Variables[scope]['local_scope'][scope_name][variable] = [value,data_type]
			return
		
	def assign_value(self,var,val,scope='global',scope_name='') :
		if scope == 'global' :
			if not self.is_var_exist(var) :
				print("Variable {} is not defined ".format(var))
				return
			_,dt = self.Variables[scope][var]
			self.Variables[scope][var] = [val,dt]
			return
		if var in self.Variables[scope]['local_scope'][scope_name].keys() :
			_,dt  = self.Variables[scope]['local_scope'][scope_name][var]
			self.Variables[scope]['local_scope'][scope_name][var] = [val,dt]
			return
		return self.assign_value(var,val,scope='global')
		
	def compute_index(self,index,scope='global',scope_name='',type='int') :
		if index :
			if self.is_number(index) :
				return int(index)
			has_op = False
			for op in self.arithmetic_op :
				if op in index :
					has_op  = True
					break
			if has_op :
				tree = ExpressionTree(Variables=self.Variables,scope=scope,scope_name=scope_name,obj=self)
				tree.buildTree(index)
				return_value = tree.computeTree(tree.head)
				if type == 'int' :
					return int(return_value)
				if type == 'float' :
					return float(return_value)
			return_value = self.fetch_var_value(index,scope=scope,scope_name=scope_name)
			if type == 'int' :
				return int(return_value)
			if type == 'float' :
				return float(return_value)
			
	def get_arr_element(self,name,index,scope='global',scope_name="") :
		if type(index) == type('a') :
			index = self.compute_index(index,scope=scope,scope_name=scope_name)
		elements,_ = self.fetch_variable(name,scope=scope,scope_name=scope_name)
		if index < len(elements) :
			return elements[index]
		print("index {} out of range for array of size {}".format(index,len(elements)))
		
	def has_arithmetic_op(self,line) :
		for op in self.arithmetic_op :
			if op in line :
				return True
		return False
		
	def process_array(self,line,scope='global',scope_name='') :
		if type(line) == type('a') :
			line = line.split()
		#int_arr arr[] = (1,2,3,4,5,6)
		#arr[6] = 7
		#arr = (1,2,3,4,5)
		if len(line) == 1 :
			#arr[i]
			# i E W
			if not self.get_arr_index(line[0]) :
				return
			name,index,_ = self.get_arr_index(line[0])
			elements = []
			data_type = None
			line_op = condition_split(index,'any')
			has_op = False
			if len(line_op) > 1:
				index = self.compute_index(index,scope=scope,scope_name=scope_name)
				has_op = True
			elements,data_type = self.fetch_variable(name,scope=scope,scope_name=scope_name)
			if not has_op :
				if not index in self.int_literals :
					if self.check_variable_name(index) :
						index = self.fetch_var_value(index,scope=scope,scope_name=scope_name)
			index = int(index)
			if index < len(elements) :
				return elements[index],data_type
			print("error index {} out of range in array {}".format(index,line[0]))
			return
		
		if len(line) == 4 :
			#int_arr arr[] = (1,2,3,4,5,6,7)
			#float_arr arr[] = (1.23,9.9)
			#int arr[5+1] = (1,2,3,4,5)
			if line[0] in self.ArrayDataTypes :
				data_type = line[0]
				name = line[1]
				is_arr= self.get_arr_index(name)
				if not is_arr :
					return
				name,index,_ = self.get_arr_index(name)
				index = self.compute_index(index,scope=scope,scope_name=scope_name)
				temp_elements = ''
				count = 0
				for e in line[-1] :
					if e == '('  :
						count += 1
						continue
					if e == ')' :
						count += 1
						continue
					temp_elements += e 
				elements_delta = []
				for e in temp_elements.split(',') :
					if self.is_number(e) :
						elements_delta.append(e)
						continue
					elements_delta.append(self.fetch_var_value(e))
				#fetched values of the element	
				if not count >= 2 :
					print("Syntax error {}".format(line))
					return
				if index :
					if index != len(elements_delta) :
						print("index {} does not match the number of elements in the array {}".format(index,elements))
						return
				if scope == 'local' :
					self.Variables['local']['local_scope'][scope_name][name] = [elements_delta,data_type]
					return
				data_type = data_type
				self.Variables['global'][name] = [elements_delta,data_type]
				return
		
		if len(line) == 3 :
			# y = arr[i]
			#arr[i] = y
			# t = arr[]
			#res = 2+arr[i]
			y = line[0]
			arr = line[2]
			clean_line = condition_split(arr,'any')
			have_arr = False
			for element in clean_line :
				if self.get_arr_index(element) :
					have_arr = True
					break				
			if not have_arr :
				return
			has_op = True
			if len(clean_line) == 1 :
				has_op = False
				
			if has_op :
				#var = arr[i+k]*arr[h+k]
				tree = ExpressionTree(Variables=self.Variables,scope=scope,scope_name=scope_name,obj=self)
				tree.buildTree(arr)
				val  = str(tree.computeTree(tree.head))
				#if y is an array then this will be executed
				if self.get_arr_index(y) :
					name,index_y,_ = self.get_arr_index(y)
					return_value = condition_split(index_y,'any')
					if len(return_value) > 1 :
						index_y = self.compute_index(index_y,scope=scope,scope_name=scope_name)
					if self.is_number(str(index_y)) :
						pass
					else :
						index_y = self.fetch_var_value(index_y,scope=scope,scope_name=scope_name)
					index_y = int(index_y)
					self.assign_array(name,val,index=index_y,scope=scope,scope_name=scope_name)
					return True
				#if y is a variable then this line will be executed
				self.assign_value(y,val,scope=scope,scope_name=scope_name)
				return True
			
			if  self.get_arr_index(arr) or  self.get_arr_index(y) :
				pass
			else :
				return
			#arr[i] = y
			if self.get_arr_index(y)  :
				name,index,_ = self.get_arr_index(y)
				index = self.compute_index(index,scope=scope,scope_name=scope_name)
				elements,dt  = self.fetch_variable(name,scope=scope,scope_name=scope_name)
				
				if self.check_variable_name(arr) :
					arr,_ = self.fetch_variable(arr,scope=scope,scope_name=scope_name)
				
				elif self.get_arr_index(arr) :
					#arr[i] = arr[j]
					name_j,j_index,_= self.get_arr_index(arr)
					j_index = self.compute_index(j_index,scope=scope,scope_name=scope_name)
					j_arr = self.get_arr_element(name_j,j_index,scope=scope,scope_name=scope_name)
					print(j_index)
					self.assign_array(name,j_arr,index=index,scope=scope,scope_name=scope_name)
					return
				self.assign_array(name,arr,index=index,scope=scope,scope_name=scope_name)
				return
			#y = arr[i]
			return_val = self.process_array(arr,scope=scope,scope_name=scope_name)
			if not return_val :
				return
			data_type = return_val[1].split('_')[0]
			self.assign_value(y,return_val[0],scope=scope,scope_name=scope_name)
			return
			
	def is_array(self,line) :
		for l in line :
			if l.split() == [] :
				continue
			if self.get_arr_index(l.split()[0]) :
				return True
		return False
		
	def is_function(self,line) :
		line = line.split('=')
		if len(line) == 1 :
			return True
		line  = line[1]
		# a = pow(2,3)
		if line.split('(')[0] in self.functions :
			return True
		if line.split('(')[0] in self.func_body :
			return True
		return False
		
	def check_number_literals(self,var,value) :
		for v in value :
			if v == '.' :
				continue
			if v == '-' :
				continue
			if not v in self.int_literals :
				print(self.value_error.format(value,var))
				return False
		return True
		
	def check_char_literals(self,var,value) :
		if value[0] != "'" or value[-1] != "'"  :
			print(self.value_error.format(value,var))
			return
		for v in value :
			if v in self.symbols :
				continue
			if not v.lower() in self.char_literals + self.int_literals :
				print(self.value_error.format(value,var))
				return False
		return True
		
	def format_char_literals(self,chars) :
		line = ''
		for i,char in enumerate(chars) :
			if i + 1 == len(chars) :
				line += char
				break
			line += char
			line += ' '
		return line
	
	def process_variable(self,line,scope='global',scope_name='') :
		if self.is_array(line.split('=')) :
			return
		if self.is_function(line) :
			return
		line = line.split()
		if len(line) <= 2 :
			return
		N = len(line)
		#DataTypes Var_name = Value
		#Var_name = value
		if line[0] == 'func':
		    return
		if line[0] in self.DataTypes :
			N = 4
		else :
			N = 3
		
		if N == 4 :
			data_type = line[0]
			if data_type in self.ArrayDataTypes :
				return
			
			name = line[1]
			value = line[3]
			if line[2] != '=' :
				print("Syntax Error from process_variable unit {}".format(line))
				return
			if not self.check_variable_name(name) :
				return
			
			if data_type in ['int','float'] :
				has_operator = False
				val = condition_split(value,'any')
				if len(val) > 1 :
					has_operator = True
				del val
				if has_operator :
					value = str(self.compute_index(value,scope=scope,scope_name=scope_name,type='float'))
				if not self.check_number_literals(name,value) :
					return
				self.define_variable(name,value,data_type,scope=scope,scope_name=scope_name)
				return True
				
			if data_type == 'chars' :
				value = self.format_char_literals(line[3:])
				if not self.check_char_literals(name,value) :
					return
				self.define_variable(name,value,data_type,scope=scope,scope_name=scope_name)
				return True
				
		if N == 3 :
			# a = 3
			if scope == 'local' :
				if scope_name == '' :
					 print("Error from process_variable unit please provide the name of the scope ")
					 return
				name = line[0]
				value = line[2]
				val = condition_split(value,'any')
				if len(val) > 1 :
					have_array = False
					for each_ele in val :
						if self.get_arr_index(each_ele) :
							return
				else :
					if self.get_arr_index(val[0]) :
						return
				
				if name in self.Variables[scope]['local_scope'][scope_name].keys() :
				   _,data_type = self.Variables[scope]['local_scope'][scope_name][name]
				   
				   if data_type in ['int','float'] :
				   	has_operator = False
				   	val = condition_split(value,'any')
				   	if len(val) > 1 :
				   		has_operator = True
				   	if has_operator :
				   	    value = str(self.compute_index(value,scope=scope,scope_name=scope_name,type='float'))
				   	if not self.check_number_literals(name,value) :
				   		return    
				   	self.assign_value(name,value,scope=scope,scope_name=scope_name)
				   	return True
				   if data_type == 'chars' :
				       value = self.format_char_literals(line[2:])
				       if not self.check_char_literals(name,value) :
				       	return
				       self.assign_value(name,value,scope=scope,scope_name=scope_name)
				       return True
				else :
					 line = self.format_char_literals(line)
					 return self.process_variable(line,scope='global')
					 if name in self.Variables['global'].keys() :
					 	_,data_type = self.Variables['global'][name]
					 	if data_type in ['int','float'] :
					 		has_op = False
					 		val = condition_split(value,'any')
					 		if len(val) > 1 :
					 			have_op = True
					 		if has_op :
					 			value = str(self.compute_index(value,scope=scope,scope_name=scope_name,type='float'))
					 		if not self.check_number_literals(name,value) :
					 			return
					 		self.assign_value(name,value,scope=scope,scope_name=scope_name)
					 		return True
					 	if data_type == 'chars' :
					 		value = self.format_char_literals(line[2:])
					 		if not self.check_char_literals(name,value) :
					 			return
					 		self.assign_value(name,value,scope=scope,scope_name=scope_name)
					 		return True	
					 print("Error assigning value before defining a variable {}".format(line))
					 return
			if scope == 'global' :
				name = line[0]
				value = line[2]
				if line[1] != '=' :
					print("Syntax Error {}".format(line))
					return	
				if name in self.Variables[scope].keys() :
					_,data_type = self.Variables[scope][name]
					if data_type in ['int','float'] :
						has_operator = False
						val = condition_split(value,'any')
						if len(val) > 1 :
							has_operator = True
						del val
						if has_operator :
							value = str(self.compute_index(value,scope=scope,scope_name=scope_name,type='float'))
						if not self.check_number_literals(name,value) :
							return
						self.assign_value(name,value,scope=scope,scope_name=scope_name)
						return True
					if data_type == 'chars' :
						value = self.format_char_literals(line[2:])
						if not self.check_char_literals(name,value) :
							return
						self.assign_value(name,value,scope=scope,scope_name=scope_name)
						return True
				else :
				 	print("Error variable is not defined {}".format(line))
				 	return
	
	def output_func(self,line,scope='global',scope_name='') :
		if '(' in line :
			line = line.split()
			if len(line) > 1 :
				return
			l = ""
			for i in line :
				l += i
			line = l
			del l
			expr = ''
			has_found = False
			for i in range(len(line)-1) :
				if line[i] == '(' :
					has_found = True
					continue
				if has_found :
					expr += line[i]
			if expr == '' :
				return
			if expr[0] == "'" and expr[-1] == "'" :
				print(expr)
				return
			expr = expr.split(',')
			if len(expr) >= 1 :
				for v in expr :
					has_op = False
					for op in ['*','/','+','-'] :
						if op in v :
							has_op = True
							break
					if has_op :
						if " " in v :
							e = v.split()
							expr_ = ''
							for i in e :
								expr_ += i
							v = expr_
						tree = ExpressionTree(Variables=self.Variables,scope=scope,scope_name=scope_name)
						tree.buildTree(v)
						print(tree.computeTree(tree.head))
						continue
					for i in v :
						if i in self.alpha :
							if scope == 'global' :
								try :
									val,_ = self.Variables[scope][v]
									print(val,' ')
									break
								except :
									pass
							if scope == 'local' :
								if v in self.Variables['local']['local_scope'][scope_name].keys():
									val,_ = self.Variables[scope]['local_scope'][scope_name][v]
									print(val," ")
									break
								if v in self.Variables['global'].keys() :
									v,_  = self.Variables['global'][v]
									print(v)
									break
						else :
							print(v," ")
							break
			return
	
	def output(self,line,scope="global",scope_name="") :
		pass
		
	def has_comment(self,line) :
		if self.comment_syntax in line.split()[0] :
			return True
		return False
	
	def ProcessLine(self,line,scope='',scope_name='') :
		if self.has_comment(line) :
			return	
		self.process_variable(line,scope=scope,scope_name=scope_name)
		self.process_Func(line,scope=scope)
		self.process_function(line,scope=scope,scope_name=scope_name)
		self.output_func(line,scope=scope,scope_name=scope_name)
		self.process_array(line,scope=scope,scope_name=scope_name)
	
	def ProcessLines(self,lines) :
		if type(lines) == type('a') :
			if '\n' in lines :
				lines = lines.split('\n')
		count = 0
		i = 0
		loop_count = 0
		N = len(lines)
		while i < N :
			line = lines[i]
			if line.split() == [] :
				i = i + 1
				continue
			if line.split()[0] == 'terminate' :
				return
			if line.split()[0] == 'Func' :
				name,_,_ = self.get_func_param(line)
				if len(name.split()) > 1 :
					name = name.split()[1]
				self.Variables['local']['local_scope'][name] = {}
				Func = FPU(compiler=self,scope='local',scope_name=name)
				self.functions[name] = Func
				_,pos = Func.buildTree(lines[i:],indent=self.indent)
				i = i + pos
				continue
			if line.split()[0] in self.loop_types :
				self.Variables['local']['local_scope']['for'+str(loop_count)] = {}
				loop = LoopProcessingUnit(self.Variables,scope_name='for'+str(loop_count),indent=self.indent,process_variable=self.process_variable,output_func=self.output_func,obj=self)
				pos = loop.buildTree(lines[i:])
				loop.executeTree(node=loop.loop_node)
				i = i + pos
				continue
			if line.split()[0]  == 'if' :
				self.Variables['local']['local_scope']['if_0'] =  {}
				cpu = CPU(scope='local',scope_name='if_0',compiler=self)
				cpu.indent = 0
				pos = cpu.buildTree(lines[i:],indent=self.indent)
				cpu.executeTree(cpu.head)
				i  = i + pos
				continue
			self.ProcessLine(line,scope='global')
			i= i + 1
		return
		
	def compile(self,lines) :
		pass
		
#compiler = Compiler()
Lines = """
int a = 2
chars name = 'Pinku Khare'
func f(x) = x*x+2+4/3
float s = a*2
int res = 0
name = 'Prakamya Khare'
int num = s*a+3*a
print("Prakamya")
for i = 1 to a :
    int a = 2
    for j = 1 to 10 :
        func g(x) = 8*x
        a = g(j)
        print(a)
        for k = 1 to 3 :
            res = res+1
            a = f(k)
            print(a)
        a = a+2
        res = res+1
        print("Pinku",j)
    print(a)
print(a)
print(name)
print('res',res)
for i = 1 to 5 :
    print(i)
print(s)
res = f(3)
print(res)
func pow(n) = n*n
int res = 0
res = pow(5)
print(res)
func multivar(x,y) = x+x*y+y
res = multivar(3,4)
print(res)
res = f(3)
func two(x) = pow(x)/f(x)
print(res)
res = two(2)
print(res)
print('eeeee')
func fun(x) = 2*x
func hh(x) = fun(x)/pow(x)
int ss = 0
ss = pow(2)
print(ss)
ss = hh(2)
print(ss)
func gg(x) = 3*fun(x)*multivar(x,x)
ss = gg(2)
print(ss)

Func demo(x)  :
    int a = 0
    for i = 1 to x :
        a = a+i
    return a
ss = demo(2)
print('ss')
print(ss)
print('ss')
print(ss)
print('name')
""" 

L1 = """
func g(x) = x*x
func h(x) = 9*x
Func f(x) :
    int res = 1
    for i = 1 to x :
        res = res*i
    print('res')
    print(res)
    return res
Func pow(x,r) :
    float res = x-0
    int b = r-1
    for i = 1 to b :
        res = res*x
    return res

int a = 0
int res = 0
res = f(10)
print(res)
a = h(44)
print(a)
res = 2*h(a)/f(2)
print(res)
res = pow(2,5)
print(res)
print('here')
for i = 1 to 10 :
    print('here')
    for j = 1 to 10 :
        print(j)
"""

L2 = """
Func f(x,y) :
    int res = 0
    for i = x to y :
        res = res+i
    print(res)
    for i = x to y :
        print("here")
        for j = x to y :
            for k = 1 to 4 :
                print(k)
            print('nes')
            res = res+j
    print(res)
    return res

int val = 0
val = f(1,10)
print(val)

Func pow(x,r) :
    int res = x-0
    int r = r-1
    for i = 1 to r :
        res = res*x
    return res
func f(x) = 2*x
func g(x) = 3*x
float res = 0.0
res  = f(2)+f(3)
print(res)

"""

L3 = """
func f(x) = 2*x
func g(x) = 3*x
float res = 0.0
float x = 3
res = g(3)
float res1 = 0.0
res1 = f(3)
float a = 0.0
res1 = res/res1
print(res1)
res = f(2)+g(3)
"""
L4 = """
func s(k) = k+1
Func pow(x,r) :
    int res = x-0
    r = r-1
    int v = 0
    v = s(x)
    for i = 1 to r :
        res = res*x
    return res
func f(x) = 2*x
func g(x) = 4*x
int res = 0.0
int y = 4
int h = 2
res = f(4)+g(2)
print(res)
int res1 = f(res)+g(res)
print(res)
print(res1)
int a = 0
a = pow(2,5)
print(a)
int x = 2
int r = 2
int value = x*3/pow(x,r)
print(value)
int value = pow(x,r)*pow(x,r)/g(x)+f(4)
print(value)
int a = 3
int b = 4
if a |= b :
    for i = 1 to 10 :
        print(i)
        print('1')
        if i == 8 :
            print('i == 8')
        else :
            print(i)
    a = pow(a,a)
    print(a)
    print('True')
else :
    a = pow(2,2)
    print(a)
    print('False')

print('here')
"""
L5 = """
int a = 5
for i = 1 to 5 :
    if i == 4 :
        if i == 2 :
            print('here')
            print('22222')
        print('if')
    else :
        if i == 3 :
            print('aaaaa')
        print('else')
        if i == 3 :
            print('trueeeee')
    print(i)
print('here')

Func condt(x) :
    int istrue = 0
    if x == 1 :
        istrue = 1
        print('true')
    return istrue

int val = 0
val = condt(1)
print(val)

int a = 3
int n = 4
if a |= n :
    if a < n :
        print(a)
        print('a')
    else :
        print(n)
        print('m'n)
else :
    print('aaa')
print('heeeeeey')
"""

L6 = """
for i = 1 to 10 :
    if i == 5 :
        if i |= 4 :
            print('true')
        print('isone')
        print(i)
    else :
        if i == 2 :
            print('isone')
            print(i)
            print('i==2')
        else :
            print(i)
    print(i)

print('heyitsworking')
"""
L7 = """
int x = 2

Func poww(x,r) :
    int res = x-0
    int r = r-1
    for i = 1 to r :
        res = res*x
    print(res)
    return res

Func pow(x,r) :
    if r == 0 :
        print('hereso')
        return 1
    int res = x-0
    int val = 0
    if r < 0 :
        res = pow(x,-r)
        res = 1/res
        return res
    if r == 1 :
        print('here')
        return x
    if r == 0 :
        print('hereo')
        return 1
    r = r-1
    for i = 1 to r :
        res = res*x
    return res        

float val = 0
val = pow(2,-1)
print(val)
for i = 1 to 10 :
    print(i)
int i = 2
if i |= 1 :
    print('yes')
"""
L9 = """
Func pow(x,r) :
    if r == 0 :
        return 1
    if r == 1 :
        print('hereinsideif')
        return x
    print('hereout')
    int res = x-0
    r = r-1
    for i = 1 to r :
        res = res*x
    return res
chars name = 'Prakamya Khare'
int res = 0
int e = 0
res = pow(2,e)
print(res)

Func fact(x) :
    if x == 1 :
        return x
    int res = 0
    int a = x-0
    x = x-1
    res = a*fact(x)
    return res

int fac = 0
fac = fact(6)
print(fac)
int n = 5
int_arr arr[n] = (1,2,3,2324,5)
print(arr)
int y = 0
int i = 2
y = arr[i*2]
print(y)
int j = 1
arr[j+j+j] = arr[j+1]
print(arr)
for i = 1 to n :
    print(arr[i])
Func mean(arr,n) :
    int res = 0
    for i = 1 to n :
        res = res+arr[i-1]
    res = res/n
    return res
int res = 0
res = mean(arr,n)
int k = 3
int j = 0
print(arr)
arr[j+1] = arr[k+1]*arr[k-1]
print(res)
print(arr)
if k == 1 :
    print(k)
else :
    if j == 0 :
        if k == 3 :
            print('here')
    else :
        print('else')
float a = arr[k+1]*arr[k-1]
print(a)
"""

L10 = """
func f(x,y) = 2*x+y
Func g(x,y,z) :
    float res = 0
    res = 2*x+y*z
    return res
int_arr arr[] = (1,2,3,4,5)
print(arr)
float res = 0.0
int k = 0
int l = 1
arr[k+1] = arr[k]*arr[k+2*l]
k = arr[k+1]
res = f(k,5)
print('res')
print(res)
print(arr)
res = g(3,2,3)
print(res)
"""

#print(compiler.Variables['global'])


from .LoopProcessingUnit2 import LoopProcessingUnit as LPU
from .ExpressionTree1 import ExpressionTree as ET
from .ConditonalProcessingUnit import ConditionalProcessingUnit as CPU
from .StringAlgorithm.ConditionSplit import condition_split
from .StringAlgorithm.ConditionSplitV3 import condition_split_v2 
from .StringAlgorithm.ExtractFuncNameParams import extract_name_param
from .StringAlgorithm.SplitString import split

class FuncNode() :
	
	def __init__(self,) :
		self.name = ''
		self.body = []
		self.params = ''
	
class FunctionProcessingUnit :
	
	def __init__(self,scope='',scope_name='',compiler=None,indent=0) :
		self.head = None
		self.compiler = compiler
		self.For = LPU(compiler.Variables,scope_name=scope_name)
		self.cpu = None
		self.scope = scope
		self.indent = indent
		self.nums = '0123456789'
		self.scope_name = scope_name
		self.outer_scope = None
	
	
	def has_indent(self,line,indent) :
		count = 0
		for i in line :
			if i == ' ' :
				count += 1
				continue
			break
		if count == indent :
			return True
		return False
		
	def format_line(self,line) :
		new_line = []
		for e in line :
			if e == ' ' or e == '' :
				continue
			new_line.append(e)
		return new_line
	
	def check_syntax(self,line) :
		#Func Name(a1,a2,a3) :
		#Name(a1,a2,a3)
		line = condition_split(line,' ')
		line = self.format_line(line)
		
		if len(line) == 1 :
			name,param,found = extract_name_param(line[0])
			if not found :
				print("Error not a function from FPU :- check_syntax")
				return
			if name != self.head.name :
				print("error function {} is not defined".format(name))
				return
			if not self.compiler.check_variable_name(name) :
				print("Incorrect name {} given to the function".format(name))
				return
			p = param.split(',')
			for e in p :
				if not self.compiler.check_variable_name(e) :
					print("Incorrect paramater name {}".format(e))
					return
			return True
		
		if line[0] != 'Func' :
			return
		name,param,found = extract_name_param(line[1])
		
		if not found :
			return
		if not self.compiler.check_variable_name(name) :
			print("Incorrect name {} given to the function".format(name))
			return
		p = param.split(',')
		for e in p :
			e = e.split()[1]
			if not self.compiler.check_variable_name(e) :
				print("Incorrect paramater name {}".format(e))
				return
		if line[2] != ':' :
			print("Syntax Errror {}".format(line))
			return
		return True
	
	def buildTree(self,lines,prev_node=None,indent=0) :
		node = FuncNode()
		if type(lines) == type('a') :
			lines = lines.split('\n')
		if not self.check_syntax(lines[0]) :
			return
		name,params,_= self.compiler.get_func_param(lines[0])
		node.name = name.split()[1]
		node.params = params
		if not prev_node :
			for param in params.split(',') :
				data_type = param.split()[0]
				variable = param.split()[1]
				self.compiler.define_variable(variable,"",data_type,scope='local',scope_name=self.scope_name)
		if prev_node :
			prev_node.body.append(node)
		else :
			self.head = node
		i = 1
		N = len(lines)
		while i < N :
			line = lines[i]
			if line.split() == [] :
				i = i + 1
				continue
			if not self.has_indent(line,indent) :
				break
			if line.split()[0] == 'Func' :
				_,pos = self.buildTree(lines[i:],prev_node=node,indent=indent+self.compiler.indent)
				i = i + pos
				continue
			if line.split()[0] == 'for' :
				for_obj = LPU(self.compiler.Variables,scope=self.scope,scope_name=self.scope_name,indent=self.compiler.indent,process_variable=self.compiler.process_variable,output_func=self.compiler.output_func,obj=self.compiler,is_func=True)
				node.body.append(for_obj)
				for_obj.indent = 0
				pos = for_obj.buildTree(lines[i:],indent=indent+self.compiler.indent)
				i = i + pos
				continue
			if line.split()[0] == 'if' :
				cpu = CPU(scope=self.scope,scope_name=self.scope_name,compiler=self.compiler)
				cpu.indent = 0
				pos = cpu.buildTree(lines[i:],indent=indent+self.compiler.indent)
				node.body.append(cpu)
				i = i + pos
				if not self.cpu :
					self.cpu = cpu
				continue
			node.body.append(line)
			i = i + 1
			continue
		return 0,i
		
	def is_def(self,var) :
		if var in self.compiler.Variables['local']['local_scope'][self.scope_name].keys() :
			return True
		if var in self.compiler.Variables['global'].keys() :
			return True
		return False
	
	def minus_minus_plus(self,num) :
		if len(num) >= 3 :
			if num[0] == '-' and num[1] == '-' :
				return num[2:]
		return num
	
	def fetchValue(self,var) :
		if var in self.compiler.Variables['local']['local_scope'][self.scope_name].keys() :
			return self.compiler.Variables['local']['local_scope'][self.scope_name][var]
		if var in self.compiler.Variables['global'].keys() :
			return self.compiler.Variables['global'][var]
		print("Error variable {} is not defined ".format(var))
		return
		
	def print_variable(self,name,scope) :
		print(self.compiler.Variables['local']['local_scope'][scope][name])
	
	def format_negative_variables(self,variable) :
		if variable == "" :
			return
		#r = -r
		# -r =-val
		if variable[0] == '-' :
			return variable[1:]
		return
	
	def executeTree(self,line,has_scope=False) :
		# y = f(2)
		"""
		Func f(x) :
			int a = 0
			for i =1 to 10 :
				a = a + 1
			return a*x
		
		res = 2+f(2)
		"""
		
		if type(line) == type('a') :
			line = line.split()
			
		
		if len(line) == 1 :
			#f(2)
			#f(2,3)
			#f(x,y)
			#param_val = [2,3]
			#param_name = [x,y]

			name,param,_ = extract_name_param(line[0])
			params_val = split(param,',')
			params_name = self.head.params.split(',')

			for key,val in zip(params_name,params_val) :
				key = key.split()[1]
				is_negative = False

				if self.compiler.is_function(val) :
					val = self.compiler.execute_Functions(val,scope=self.scope,scope_name=self.scope_name)
					self.compiler.assign_value(key,val,scope=self.scope,scope_name=self.scope_name)
					continue
					
				if self.compiler.get_arr_index(val) and not '.' in val :
					try :
						val,_ = self.compiler.process_array(val,scope=self.scope,scope_name=self.scope_name)
					except :
						val,_ = self.compiler.process_array(val,scope=self.scope,scope_name=self.outer_scope)
					self.compiler.assign_value(key,val,scope=self.scope,scope_name=self.scope_name)
					continue
					
				
				if self.compiler.is_number(val) :
					self.compiler.assign_value(key,val,scope='local',scope_name=self.scope_name)
					continue
					
				if len(condition_split(val,'any')) > 1 :
					val = self.compiler.compute_index(val,scope=self.scope,scope_name=self.scope_name)
					self.compiler.assign_value(key,val,scope='local',scope_name=self.scope_name)
					continue
				
				if '.' in val :
					val_ = self.compiler.line_to_execute(val,'local',self.scope_name,'accessing_attributes')
					if val_ == None :
						if self.outer_scope :
							val_ = self.compiler.line_to_execute(val,'local',self.outer_scope,'accessing_attributes')
					self.compiler.assign_value(key,val_,scope='local',scope_name=self.scope_name)
					continue
				
				if self.format_negative_variables(val) :
					val = self.format_negative_variables(val)
					is_negative = True
				
				if self.outer_scope == self.scope_name :
					#recursion case
					if val in self.compiler.Variables['local']['local_scope'][self.scope_name].keys() :
						data,dt = self.compiler.fetch_variable(val,scope='local',scope_name=self.scope_name)
						if data != '' or data != None :
							if is_negative :
								data = '-' + data
								data = self.minus_minus_plus(data)
							self.compiler.assign_value(key,data,scope='local',scope_name=self.scope_name)
							continue
					if val in self.compiler.Variables['global'].keys() :
						data,dt = self.compiler.fetch_variable(val,scope='global')
						if is_negative :
							data = '-' + data
							data = self.minus_minus_plus(data)
						self.compiler.assign_value(key,data,scope=self.scope,scope_name=self.scope_name)
						continue
						
				
				if self.outer_scope :
					
					if val in self.compiler.Variables['local']['local_scope'][self.outer_scope].keys() :
						data,dt = self.compiler.fetch_variable(val,scope=self.scope,scope_name=self.outer_scope)
						if data == '' or data == None :
							pass
						else :
							if is_negative :
								data = '-' + data
								data = self.minus_minus_plus(data)
							self.compiler.Variables['local']['local_scope'][self.scope_name][key] = [data,dt]
							continue
				
				if val in self.compiler.Variables['global'].keys() :
					v,dt = self.compiler.Variables['global'][val]
					if v == '' or v == None :
						continue
					if is_negative :
						v = '-' + v
						v = self.minus_minus_plus(v)
					self.compiler.assign_value(key,v,scope='local',scope_name=self.scope_name)
					continue

				"""
				if val in self.compiler.Variables['local']['local_scope'][self.scope_name].keys() :
					data,dt = self.compiler.fetch_variable(val,scope='local',scope_name=self.scope_name)
					if data != '' or data != None :
						if is_negative :
							data = '-' + data
							data = self.minus_minus_plus(data)
						self.compiler.Variables['local']['local_scope'][self.scope_name][val] = [data,dt]
						continue
				"""
				

				"""	
				if self.format_negative_variables(val) :
					name = self.format_negative_variables(val)
					v,dt = self.fetchValue(name)
					v = '-'+v
					self.compiler.assign_value(key,v,scope='local',scope_name=self.scope_name)
					continue
				"""
				print("error param {} is not given any defined value or variable {}".format(key,val))
				return
			#print(self.compiler.Variables['local']['local_scope'][name]
			v = self.execute(self.head)
			if type(v) == type([1]) :
				if len(v) == 1 :
					return v[0]
					
			return v

		y = line[0]
		if line[1] != '=' :
			print("Invalid Syntax from FPU {}".format(line))
			return
		fun = line[2]
		
		name,param,found = extract_name_param(fun)
		
		if name != self.head.name :
			print("Error function {} is not defined".format(name))
			return

		params_val = split(param,',')
		params = self.head.params.split(',')
		#print(self.compiler.Variables['local']['local_scope'][name])
		#f(x,y)
		#f(1,2)
		#params_val = [1,2]
		#params = [x,y]
		Vars = self.compiler.Variables['local']['local_scope'][self.scope_name]
		
		for val,key in zip(params_val,params) :
			key = key.split()[1]
			is_negative = False
			
			if self.compiler.is_function(val) :
				
				val = self.compiler.execute_Functions(val,scope=self.scope,scope_name=self.scope_name)
				self.compiler.assign_value(key,val,scope=self.scope,scope_name=self.scope_name)
				continue
				
			if self.compiler.get_arr_index(val) and not '.' in val :
				try :
					val,_ = self.compiler.process_array(val,scope=self.scope,scope_name=self.scope_name)
				except :
					if self.outer_scope == 'global' :
						val,_ = self.compiler.process_array(val,scope='global')
					else :
						val,_ = self.compiler.process_array(val,scope=self.scope,scope_name=self.outer_scope)
				self.compiler.assign_value(key,val,scope=self.scope,scope_name=self.scope_name)
				continue
				
			if self.compiler.is_number(val) :
				self.compiler.assign_value(key,val,scope='local',scope_name=self.scope_name)
				continue
				
			
			if len(condition_split(val,'any')) > 1 :
				val = self.compiler.compute_index(val,scope='local',scope_name=self.scope_name)
				self.compiler.assign_value(key,val,scope='local',scope_name=self.scope_name)
				continue
				
			if '.' in val :
				val_ = self.compiler.line_to_execute(val,'local',self.scope_name,'accessing_attributes')
				if val_ == None :
					if self.outer_scope :
						val_ = self.compiler.line_to_execute(val,'local',self.outer_scope,'accessing_attributes')
				self.compiler.assign_value(key,val_,scope='local',scope_name=self.scope_name)
				continue
				
			if self.format_negative_variables(val) :
				val  = self.format_negative_variables(val)
				is_negative = True
				
			if self.outer_scope == self.scope_name :
				if val in Vars.keys() :
					data,dt = self.compiler.fetch_variable(val,scope='local',scope_name=self.scope_name)
					if is_negative :
						data = '-' + data
						data = self.minus_minus_plus(data)
					self.compiler.Variables['local']['local_scope'][self.scope_name][key] = [data,dt]
					#self.compiler.assign_value(key,data,scope='local',scope_name=self.scope_name)
					continue
				if val in self.compiler.Variables['global'].keys() :
					data,dt = self.compiler.fetch_variable(val,scope='global')
					if is_negative :
						data = '-' + data
						data = self.minus_minus_plus(data)
					self.compiler.Variables['global'][key] = [data,dt]
					#self.compiler.assign_value(key,data,scope='local',scope_name=self.scope_name)
					continue
			
			
			if self.outer_scope :
				if val in self.compiler.Variables['local']['local_scope'][self.outer_scope].keys() :
					data,dt = self.compiler.Variables['local']['local_scope'][self.outer_scope][val]
					if data != '' or data != None :
						if is_negative :
							data = '-' + data
							data = self.minus_minus_plus(data)
						self.compiler.Variables['local']['local_scope'][self.scope_name][key] = [data,dt]
						continue
						
			
			if val in self.compiler.Variables['global'].keys() :
				v,dt = self.compiler.Variables['global'][val]
				if v == '' or v == None :
					pass
				else :
					if is_negative :
						v = '-' + v
						v = self.minus_minus_plus(v)
						v = str(float(v))
					self.compiler.Variables['local']['local_scope'][self.scope_name][key] = [v,dt]
					continue
			"""
			
			if val in self.compiler.Variables['local']['local_scope'][self.scope_name].keys() :
				data,dt = self.compiler.fetch_variable(val,scope=self.scope,scope_name=self.scope_name)
				if data == '' or data == None :
					pass
				else :
					if is_negative :
						data ='-' + data
						data = self.minus_minus_plus(data)
						data = str(float(data))
					self.compiler.Variables['local']['local_scope'][self.scope_name][val] = [data,dt]
					continue
			"""
			
				
			#if val in self.compiler.Variables['local']['local_scope'][self.scope_name].keys() :
				#v,dt = self.compiler.Variables['local']['local_scope'][self.scope_name][val]
				#self.assign_value(key,v,scope='local',scope_name=self.scope_name)
				#continue
				
			


			"""
			if self.format_negative_variables(val) :
				v = self.format_negative_variables(val)
				v,dt = self.compiler.fetch_variable(v,scope='local',scope_name=self.scope_name)
				v = '-'+v
				self.compiler.Variables['local']['local_scope'][self.scope_name][val] = [v,dt]
			"""
			print("Error parameter {} is not given any value that is defined {} ".format(key,val))
			
			
		v = self.execute(self.head)
		if type(v) == type(['1']) :
			if len(v) == 1 :
				return v[0]
		return v
		
	def execute(self,node=None) :
		if node :
			for line in node.body :
				if type(line) == type('a') :
					if line.split()[0] == 'return' :
						l = line.split()
						if len(l) == 1 :
							return 1
						vals = l[1]
						return_list = []
						if len(condition_split_v2(vals,'any')) > 1 :
							#has arithmetic op
							val = self.compiler.compute_index(vals,scope='local',scope_name=self.scope_name,type='float')
							return val
						if self.compiler.is_function(vals) :
							name,params,_ = self.compiler.get_func_param(vals)
							if name in self.compiler.functions.keys() :
								func = self.compiler.functions[name]
								func.outer_scope = self.scope_name
								return func.executeTree(vals)
							if name in self.compiler.func_body.keys() :
								return self.compiler.process_functions(vals,scope=self.scope,scope_name=self.scope_name,exe=True)
						vals = vals.split(',')
						for val in vals :
							if self.compiler.is_number(val) :
								return_list.append(val)
								continue
							return_list.append(self.fetchValue(val)[0])
						return return_list
				if type(line) == type(self.For) :
					line.executeTree(line.loop_node)
					continue
				if type(line) == type(self.cpu) :
					v = line.executeTree(line.head)
					if v :
						return v
					continue
				if type(line) == type(node) :
					v = self.execute(line)
					if v :
						return v
					continue
				self.compiler.ProcessLine(line,scope='local',scope_name=self.scope_name)



def get_num(line,ops) :
	N = len(line)
	offset = 0
	element = ''
	for i in range(N) :
		if not line[i] in ops :
			element += line[i]
		else :
			offset = i
			break
	return element,offset

def csp(line,op='+') :
	operators = ['+','-','*','/']
	count_ob = 0
	count_cb = 0
	i = 0
	N = len(line)
	elements = []
	flag = False
	element = ''
	while i < N :
		if i+1 >= N :
			break
		if line[i] == '-' :
			if i - 1 == -1 :
				element += line[i]
				i = i + 1
				continue
			#*-
			#/-
			elif line[i-1] in ['*','/'] :
				element += line[i]
				i = i + 1
				continue
				
		if line[i] == '-' and line[i+1] == '-' :
			# - - = +
			#  a - - b = a + b
			a = element
			b,offset = get_num(line[i+2:],operators)
			i = i + offset + 1
			element = a + '+' + b
			
			if op == 'any' :
				elements.append(a)
				elements.append(b)
				continue
			elements.append(element)
			element = ''
			continue
			
		if line[i] == '-' and line[i+1] == '+' and line[i] == '+' or line[i+1] == '-' :
			# - + = -
			# + - = -
			a = element
			b,offset = get_num(line[i+2:],operators)
			i = i + offset + 1
			
			if op == 'any' :
				elements.append(a)
				elements.append(b)
				flag = True
				continue
			if op == '-' :
				elements.append(a)
				elements.append(b)
				flag  = True
				continue
			elements.append(element)
			continue
			
		if not line[i] in op :
			element += line[i]
			if i + 1 == N :
				elements.append(element)
				break
			i = i + 1
			continue
		else :
			if flag :
				flag = False
				element = ""
				i = i + 1
				continue
			if element :
				elements.append(element)
			element = ""
			i = i + 1
	return elements
	
line = '-7--7'
print(csp(line,'-'))




def split(line,char=',') :
	new_line = []
	found_b = False
	params = ''
	open_b = ['[','(','{']
	closing_b = [']',')','}']
	N = len(line)
		
	for i in range(len(line)) :
		if line[i] in open_b :
			params += line[i]
			found_b = True
			continue
		if line[i] in closing_b :
			params += line[i]
			found_b = False
			if i+1 == N :
				new_line.append(params)
			continue
		if found_b :
			if line[i] == ' ' :
				if i+1 == N :
					new_line.append(params)
				continue
			params += line[i]
			continue
		if line[i] == char :
			new_line.append(params)
			params = ''
			continue
		params += line[i]
		if i+1 == len(line) :
			new_line.append(params)
	return new_line
	

def get_func_param(line) :
	name = ''
	param = ''
	is_found = False
	#f(g(x))
	ob_count = 0
	N = len(line)
	for i in range(N-1) :
		if line[i] == '(' :
			if ob_count == 1 :
				param += line[i]
				continue
			ob_count += 1
			is_found = True
			continue
		if not is_found :
			name += line[i]
			continue
		param += line[i]
	
	return name,param

class InputOutputStream :
	
	def __init__(self,compiler=None) :
		self.compiler = compiler
		self.output_function_name = 'ans'
		self.input_function_name = 'ask'
		
	def execute(self,line) :
		pass
				
	def ans(self,line,scope='',scope_name='') :
		if len(line.split()) > 1 :
			return
		syntax = line.split()[0]
		name,params = get_func_param(line.split()[0])
		
		if name != self.output_function_name :
			return

		params = split(params,',')
		params_value = []
		for param in params :
			if param in self.compiler.mathematical_constants.keys() :
				params_value.append(str(self.compiler.mathematical_constants[param]))
			elif self.compiler.is_char(param) :
				params_value.append(param)
			elif self.compiler.is_function(param) :
				params_value.append(str(self.compiler.execute_Functions(param)))
			elif self.compiler.get_arr_index(param) and not '.' in param :
				value,_ = self.compiler.process_array(param,scope=scope,scope_name=scope_name)
				params_value.append(value)
			elif self.compiler.is_number(param) :
				params_value.append(param)
			elif self.compiler.check_variable_name(param) :
				value,_= self.compiler.fetch_variable(param,scope=scope,scope_name=scope_name)
				params_value.append(value)
			elif '.' in param and not self.compiler.is_a_number(param) :
				value = self.compiler.line_to_execute(param,scope,scope_name,'accessing_attributes')
				params_value.append(value)
			elif len(self.compiler.function_to_execute(param,scope,scope_name,'condition_split')) > 1 :
				params.append(str(self.compiler.compute_index(param,scope=scope,scope_name=scope_name)))
			
			else :
				print("error incorrect input to ans function :- {}".format(param))
				return
		
		parameters = ""
		for param in params_value :
			parameters += str(param) + " "
		print(parameters)
		return True
	
	def ask(self,line,scope='',scope_name='') :
		if len(line.split()) > 1 :
			return
		#ask(variable,string)
		syntax = line.split()[0]
		name,params,_ = self.compiler.get_func_param(syntax)
		if name != self.input_function_name :
			return
		params = params.split(',')
		variable = params[0]
		string = params[1] if len(params) > 1 else ""
		value = input(string)
		
		#if the variable is an array so here we have to use different approach
		if self.compiler.get_arr_index(variable) :
		 	line = variable + ' ' + '=' +'  '+ value
		 	self.compiler.process_array(line,scope=scope,scope_name=scope_name)
		 	return True
		self.compiler.assign_value(variable,value,scope=scope,scope_name=scope_name)
		return True


def split(line,char=',') :
	new_line = []
	found_b = False
	params = ''
	open_b = ['[','(','{']
	closing_b = [']',')','}']
	
	for i in range(len(line)) :
		if line[i] in open_b :
			params += line[i]
			found_b = True
			continue
		if line[i] in closing_b :
			params += line[i]
			found_b = False
			continue
		if found_b :
			params += line[i]
			continue
		if line[i] == char :
			new_line.append(params)
			params = ''
			continue
		params += line[i]
		if i+1 == len(line) :
			new_line.append(params)
	print(new_line)
	return new_line
	
#split('arr[i+1,j+1],k')


from .StringAlgorithm.ExtractFuncNameParams import extract_name_param
from .LoopProcessingUnit2 import LoopProcessingUnit as LPU
from .FunctionalProcessingUnit import FunctionProcessingUnit as FPU
from .ConditonalProcessingUnit import ConditionalProcessingUnit as CPU
from .ExpressionTree1 import ExpressionTree
from  .StringAlgorithm.ConditionSplit import condition_split

class class_node :
	
	def __init__(self,) :
		self.class_name = ''
		self.body = {}
		


class ClassProcessingUnit :
	
	def __init__(self,scope='local',scope_name='',indent=None,compiler=None) :
		self.scope = scope
		self.scope_name = scope_name
		self.indent = indent
		self.compiler = compiler
		self.ERROR_CODE = -9999
		
	
	def check_syntax(self,line) :
		line = line.split()
		#class Name :
		class_name = line[1]
		if line[2] != ':' :
			print('Invalid Syntax {}'.format(line))
			return self.ERROR_CODE
		if not self.compiler.check_variable_name(class_name) :
			print("Invalid name {}".format(line))
			return self.ERROR_CODE
		
		
		
	def check_indent(self,line) :
		count = 0
		for i in range(len(line)) :
			if line[i] != ' ' :
				break
			count += 1
		if count == self.indent :
			return True
		return False
		
	def build_class(self,lines) :
		if type(lines) == type('a') :
			lines = lines.split('\n')
		node = class_node()
		self.head = node
		i = 0
		N = len(lines)
		line = lines[i]
		self.check_syntax(line)
		node.class_name = line.split()[1]
		i = 1
		while i < N :
			line = lines[i]
			if line.split() == [] :
				i = i + 1
				continue
			if not self.check_indent(line) :
				break
			if line.split()[0] == 'Func' :
				func = FPU(scope=self.scope,scope_name=self.scope_name,compiler=self.compiler)
				name,_,_ = self.compiler.get_func_param(line)
				name = 'class_'+name.split()[1]
				self.compiler.functions[name] = func
				_,pos = func.buildTree(lines[i:],indent=2*self.indent)
				node.body[name] = func
				self.compiler.Variables['local']['local_scope'][self.scope_name][name] = func
				i = i + pos
				continue
			self.compiler.ProcessLine(line,scope=self.scope,scope_name=self.scope_name,class_obj=self,Class=self)
			i = i + 1
		
		self.compiler.Variables['local']['local_scope'][self.scope_name]['class_name'] = self
		
		return i
		
	def attribute_not_found(self,name) :
		print("Error attribute of name {} is not defined".format(name))
		return
		
def accessing_attributes(self,line,scope='',scope_name='',compiler=None) :
	
	#Object.attributes = Variable
	#Variable = Object.attribute
	#Object.attributes
	#Node node
	if len(compiler.split_space(line)) >= 4 :
		return
	
	if len(line.split()) == 2 :
		if line.split()[0] == 'return' :
			return
		
		if self == None :
			class_name,object = line.split()
			self = compiler.Variables['local']['local_scope'][class_name]['class_name']
		
		line = line.split()
		class_name,obj = line
		
		
		if obj in compiler.Variables['local']['local_scope'].keys() :
			print("Variable {} of type {} has already defined".format(obj,class_name))
			return True
			
		if class_name != self.head.class_name :
			return
		compiler.Variables['local']['local_scope'][obj] = compiler.Variables['local']['local_scope'][class_name].copy()
		compiler.Variables['local']['local_scope'][obj]['class_name'] = self
		#print(compiler.Variables['local']['local_scope']['d1'])
		return True
	
		
	if '=' in line :
		
		line = line.split('=')
		left_side,right_side = line
		object = variable = None
		value = None
		#node.value 
		if '.' in left_side :
			
			object,variable = left_side.split()[0].split('.')
			object = object.split()[0]
			variable = variable.split()[0]

			if not self :
				self = compiler.Variables['local']['local_scope'][object]['class_name']
			if not object in compiler.Variables['local']['local_scope'].keys() :
				print("Object is not defined {}".format(line[0]))
				return False

			right_side = right_side.split()[0]
			
				
			#check if the right_side is a variable ,length or a function
			if right_side[0] == "'" and right_side[-1] == "'" :
				value = right_side
			if compiler.is_a_number(right_side) :
				value = right_side
			elif len(condition_split(right_side,'any')) > 1 :
				tree = ExpressionTree(Variables=self.compiler.Variables,scope=scope,scope_name=scope_name,obj=compiler)
				tree.buildTree(right_side)
				value = tree.computeTree(tree.head)
			elif compiler.check_variable_name(right_side) :
				value = self.compiler.fetch_var_value(right_side,scope=scope,scope_name=scope_name)
			elif compiler.is_function(right_side) :
				value = compiler.execute_Functions(right_side,scope=scope,scope_name=scope_name)
			elif compiler.get_arr_index(right_side) :
				value,_ = compiler.process_array(right_side,scope=scope,scope_name=scope_name)
			elif compiler.length_syntax(right_side) :
				value = compiler.length_of_array(right_side,scope=scope,scope_name=scope_name)

			if compiler.get_arr_index(variable) :
				#variable is an array
				name,index,_ = compiler.get_arr_index(variable)
				row = col = None
				if len(index.split(',')) > 1 :
					row,col = compiler.compute_coords(index.split(','),scope=scope,scope_name=scope_name)
					compiler.assign_2darray(name,value,x=row,y=col,scope='local',scope_name=object)
					return True
				if len(condition_split(index,'any')) > 1 :
					index = compiler.compute_index(index,scope=scope,scope_name=scope_name)
					index = int(index)
					compiler.assign_array(name,value,index=index,scope='local',scope_name=object)
					return True
				if compiler.is_a_number(index) :
					pass
				elif compiler.check_variable_name(index) :
					index = compiler.fetch_var_value(index,scope=scope,scope_name=scope_name)
				index = int(index)
				compiler.assign_array(name,value,index=index,scope='local',scope_name=object)
				return True

			if variable in compiler.Variables['local']['local_scope'][object].keys() :
				compiler.assign_value(variable,value,scope='local',scope_name=object)
				return True
			print("Error class {} has no such attribute {}".format(self.head.class_name,variable))
			return False
		
		if '.' in right_side and not compiler.is_a_number(right_side) :
			#n = node.data
			right_side = right_side.split()[0]
			
			if len(right_side.split('.')) != 2 :
				return False
			
			obj,variable = right_side.split()[0].split('.')
			
			y = left_side.split()[0]
			value = None
			
			if not self :
				self = compiler.Variables['local']['local_scope'][obj]['class_name']
			#if variable is an array
			#a = arr[i] or arr[i,j]
			if not obj in compiler.Variables['local']['local_scope'].keys() :
				print('Error object of name {} has not defined'.format(obj))
				return False
			
			if self.compiler.get_arr_index(variable) :
				name,index,_ =self.compiler.get_arr_index(variable)
				if not name in self.compiler.Variables['local']['local_scope'][self.head.class_name].keys() :
					self.attribute_not_found(name)
					return False
				
				if len(index.split(',')) == 2 :
					row,col = compiler.compute_coords(index.split(','),scope=scope,scope_name=scope_name)
					elements = compiler.fetch_var_value(name,scope='local',scope_name=obj)
					value = elements[row][col]
				elif len(condition_split(index,'any')) > 1 :
					index = compiler.compute_index(index,scope=scope,scope_name=scope_name)
					elements = compiler.fetch_var_value(name,scope='local',scope_name=obj)
					value = elements[index]
				elif compiler.is_a_number(index) :
					elements = compiler.fetch_var_value(name,scope='local',scope_name=scope_name)
					value = elements[int(index)] 
				elif compiler.check_variable_name(index) :
					index = int(compiler.fetch_var_value(index,scope=scope,scope_name=scope_name))
					elements = compiler.fetch_var_value(name,scope='local',scope_name=obj)
					value = elements[index]
			
			
			elif compiler.check_variable_name(variable) :
				if not variable in compiler.Variables['local']['local_scope'][obj].keys() :
					self.attribute_not_found(variable)
					return False
				value = compiler.fetch_var_value(variable,scope='local',scope_name=obj)
			
			
			elif compiler.is_function('class_'+variable) :
				name,params,_ = compiler.get_func_param('class_'+variable)
				if name in compiler.Variables['local']['local_scope'][obj].keys() :
					func = compiler.Variables['local']['local_scope'][obj][name]
					func.outer_scope = scope_name
					value = func.executeTree('class_'+variable)
				
				elif name in compiler.Variables['local']['local_scope'][obj].keys() :
					value = compiler.process_functions('class_'+variable,scope='local',scope_name=obj,exe=True)
				
				
			if compiler.get_arr_index(y) :
				name,index,_ = compiler.get_arr_index(y)
				line = left_side + ' ' + '=' + ' ' + value
				compiler.process_array(y,scope=scope,scope_name=scope_name)
			
			if compiler.check_variable_name(y) :
				compiler.assign_value(y,value,scope=scope,scope_name=scope_name)
			return True
	if type(line) == type('a') :
		line = line.split()[0]
		
	if '.' in line and not compiler.is_a_number(line) :
		obj,var = line.split('.')
		
		if not obj in compiler.Variables['local']['local_scope'].keys() :
			return
			
		if compiler.get_arr_index(var) :
			name,index,_ = compiler.get_arr_index(var)
			if len(index.split(',')) == 2 :
				row = col =  None
				try :
					row,col = compiler.compute_coords(index.split(','),scope=scope,scope_name=scope_name)
				except :
					return
				
				elements,_ = compiler.fetch_variable(name,scope='local',scope_name=obj)
				return elements[row][col]
			try :
				index = compiler.compute_index(index,scope=scope,scope_name=scope_name)
			except :
				return
			elements = compiler.fetch_var_value(name,scope='local',scope_name=obj)
			return elements[index]
			
			#return compiler.process_array(var,scope='local',scope_name=obj)[0]
			
		if var in compiler.Variables['local']['local_scope'][obj].keys() :
			val =  compiler.Variables['local']['local_scope'][obj][var]
			return val[0]
		if compiler.is_function(var) :
			name,_,_ = compiler.get_func_param(var)
			name = 'class_'+name
			func = self.Variables['local']['local_scope'][obj][name]
			func.outer_scope = scope_name
			return func.executeTree(var)
	return False



from .StringAlgorithm.ConditionSplit import condition_split


class ConditionalNode :
	
	def __init__(self,) :
		"""
		if :- True block
		else :- False block
		
		"""
		self.syntax = []
		self.body = {'if' : [],'else' : []}
		

class ConditionalProcessingUnit :
	
	def __init__(self,scope='',scope_name='',compiler=None,indent=0) :
		self.indent = compiler.indent
		self.scope  = scope
		self.for_loop = LoopProcessingUnit(compiler.Variables,scope=scope,scope_name=scope_name)
		self.scope_name = scope_name
		self.indent  = compiler.indent
		self.compiler = compiler
		self.has_loop = False
		self.nums = '0123456789'
		self.alpha = 'abcdefghijklmnopqrstuvwxyz'
		self.syntax_error = 'Syntax error {}'
		self.is_char = False
		self.boolean_ops = ['==','|=','<=','>=','<=','<','>','≠','≤','≥','∈','∉']
		
	def check_variable(self,name) :
		if name[0] in self.alpha +'_':
			for n in range(1,len(name)) :
				if not name[n].lower() in self.nums+self.alpha +'_':
					return False
			return True
		return False
		
	def has_indent(self,line,indent) :
		count = 0
		for l in line :
			if l == ' ' :
				count += 1
				continue
			break
		if count == indent+self.indent :
			return True
		return False
		
	def check_syntax(self,line) :
		if type(line) == type('a') :
			line = line.split()
		if line[0] != 'if' :
			if line[0] == 'else' and line[1] == ':' :
				if len(line) == 2 :
					return True
				return False
			print(self.syntax_error.format(line))
			return
		 #if a  == b :
		if len(line) != 5 :
		 	return
		bool_op = line[2]
		if not bool_op in self.boolean_ops :
		 	print(self.syntax_error.format(line))
		 	return
		if line[-1] != ':' :
			print(self.syntax_error.format(line))
			return
		a = line[1]
		b = line[3]
		vars = [a,b]
		for var in vars :
		 	if self.compiler.get_arr_index(var) :
		 		continue
		 	if var[0] in self.nums :
		 		continue
		 	if self.compiler.is_char(var) :
		 		self.is_char = True
		 		continue
		 	else :
		 		if not self.check_variable(var) :
		 			print("{} Incorrect variable name {}".format(line,var))
		 			return
		return True
		 
	def buildTree(self,lines,prev_node=None,mode='if',prev_mode="",indent=0) :
		line = lines[0]
		node = None
		if mode == 'if' :
			node = ConditionalNode()
			if not self.check_syntax(line) :
				return
			node.syntax = line.split()
			if prev_node :
				prev_node.body[prev_mode].append(node)
			else :
				self.head = node
		else :
			if not self.check_syntax(line) :
				print(self.syntax_error.format(line))
				return
			node = prev_node
		i = 1
		N = len(lines)
		
		while i < N :
			line = lines[i]
			if line.split() == [] :
				i  = i + 1
				continue
			if not self.has_indent(line,indent) :
				if self.has_indent(line,indent-self.compiler.indent) :
					if line.split()[0] == 'else' :
						pass
					else :
						break
				else :
					break
			if line.split()[0] == 'if' :
				pos = self.buildTree(lines[i:],prev_node=node,mode='if',prev_mode=mode,indent=indent+self.compiler.indent)
				i  = i + pos
				continue
			if line.split()[0] == 'else' :
				pos = self.buildTree(lines[i:],prev_node=node,mode='else',prev_mode=mode,indent=indent)
				i  = i + pos
				continue
			if line.split()[0] == 'for' :
				for_loop = LPU.LoopProcessingUnit(Variables=self.compiler.Variables,indent=self.compiler.indent,scope=self.scope,scope_name=self.scope_name,process_variable=self.compiler.process_variable,output_func=self.compiler.output_func,obj=self.compiler)
				
				pos  = for_loop.buildTree(lines[i:],indent=indent)
				node.body[mode].append(for_loop)
				i = i + pos
				continue
			node.body[mode].append(line)
			i = i + 1
		return i
		
	def fetchValue(self,var) :
		
		if self.compiler.is_char(var) :
			N = len(var)-1
			var = var[1:N]
			return var
		
		if self.compiler.get_arr_index(var) :
			name,index,_ = self.compiler.get_arr_index(var)
			if len(condition_split(index,'any')) > 1 :
				index = self.compiler.compute_index(index,scope=self.scope,scope_name=self.scope_name)
			elif self.check_variable(index) :
				index = self.compiler.fetch_var_value(index,scope=self.scope,scope_name=self.scope_name)
			index = int(index)
			return self.compiler.get_arr_element(name,index=index,scope=self.scope,scope_name=self.scope_name)
			
		if self.check_variable(var) :
			if self.scope == 'local' :
				if var in self.compiler.Variables['local']['local_scope'][self.scope_name].keys() :
					v,_ = self.compiler.Variables['local']['local_scope'][self.scope_name][var]
					return v
			if var in self.compiler.Variables['global'].keys() :
				v,_ = self.compiler.Variables['global'][var]
				return v
			print("Error variable {} is not defined".format(var))
			return
		return var
		
	def check_condition(self,a,b,op='==') :
		if op == '==' :
			return a==b
		if op == '|=' or op == '≠' :
			return a != b
		if op == '<' :
			return a < b
		if op == '<=' or op == "≤" :
			return a <= b
		if op == '>' :
			return a > b
		if op == '>=' or op == "≥" :
			return a >= b
		if op == '∈' :
			if a in b :
				return True
			return False
		if op == '∉' :
			if a in b :
				return False
			return True

	def compute_return_value(self,value) :
		if len(condition_split(value,'any')) > 1 :
			val = self.compiler.compute_index(value,scope='local',scope_name=self.scope_name,type='float')
			return str(val)
		if self.compiler.is_number(value) :
			return value
		return self.compiler.fetch_variable(value,scope='local',scope_name=self.scope_name)[0]
	
	def executeTree(self,node=None) :
		if node :
			#if a == b :
			syntax = node.syntax
			a = None
			b = None
			if self.is_char :
				if not self.compiler.is_char(syntax[1]) :
					a = self.compiler.fetch_var_value(syntax[1],scope=self.scope,scope_name=self.scope_name)
				else :
					a = syntax[1]
					n = len(a)-1
					a = a[1:n]
				if not self.compiler.is_char(syntax[3]) :
					b = self.compiler.fetch_var_value(syntax[3],scope=self.scope,scope_name=self.scope_name)
				else :
					b = syntax[3]
					n = len(b)-1
					b = b[1:n]
				
			elif syntax[2] in ['∈','∉'] :
				a = str(int(float(self.fetchValue(syntax[1]))))
				b = self.fetchValue(syntax[3])
			else :
				a = float(self.fetchValue(syntax[1]))
				b  = float(self.fetchValue(syntax[3]))
			
			op = syntax[2]
			if self.check_condition(a,b,op) :
				body = node.body['if']
				for line in body :
					if type(line) == type('a') :
						if line.split()[0] == 'return' :
							if len(line.split()) == 1 :
								return line
							if self.compiler.is_function(line.split()[1]) :
								name,_,_ = self.compiler.get_func_param(line.split()[1])
								if name in self.compiler.functions.keys() :
									func = self.compiler.functions[name]
									func.outer_scope = self.scope_name
									return func.executeTree(line.split()[1])
								if name in self.compiler.func_body.keys() :
									return self.compiler.process_functions(line.split()[1],scope=self.scope,scope_name=self.scope_name,exe=True)
							if len(line.split(',')) > 1 :
								line = line.split()[1].split(',')
								return_value = []
								for var in line :
									if self.compiler.is_number(var) :
										return_value.append(var)
										continue
									return_value.append(self.compiler.fetch_variable(var,scope='local',scope_name=self.scope_name)[0])
								return return_value
							if self.has_loop :
								return line
							return self.compute_return_value(line.split()[1])
							
					if type(line) == type(node) :
						v = self.executeTree(node=line)
						if v :
							return v
						continue
					if type(line) == type(self.for_loop) :
						line.executeTree(line.loop_node)
						continue
					return_val =self.compiler.ProcessLine(line,scope=self.scope,scope_name=self.scope_name)
					if return_val == self.compiler.break_statement :
						if self.has_loop :
							return return_val
					elif return_val == self.compiler.continue_statement:
						if self.has_loop :
							return return_val
				return
			else :
				body  = node.body['else']
				if body == [] :
					return
				for line in body :
					if type(line) == type('a') :
						if line.split()[0] == 'return' :
							if len(line.split()) == 1  :
								return 'return'
							if len(line.split(',')) > 1 :
								return_value = []
								line = line.split()[1].split(',')
								for var in line :
									if self.compiler.is_number(var) :
										return_value.append(var)
										continue
									return_value.append(self.compiler.fetch_variable(var,scope='local',scope_name=self.scope_name)[0])
								return return_value
							return self.compute_return_value(line.split()[1])
							
					if type(line) == type(node) :
						val = self.executeTree(line)
						if val :
							return val
						continue
					if type(line) == type(self.for_loop) :
						line.executeTree(line.loop_node)
						continue
					return_val = self.compiler.ProcessLine(line,scope=self.scope,scope_name=self.scope_name)			
					if return_val == self.compiler.break_statement :
						if self.has_loop :
							return return_val
					elif return_val == self.compiler.continue_statement :
						if self.has_loop :
							return_val
				return
			return
from .LoopProcessingUnit2 import LoopProcessingUnit


class LoopProcessingUnit :
	
	def __init__(self,indent="    ") :
		self.indent = indent
		self.for_loop_syntax = ['=','to',':']
		self.error_code = -222
		self.while_loop_syntax = ['<,>,<=,>=',':']
		if len(self.indent) == 0 :
			self.indent = "    "
	
	def check_variable(self,var) :
		if type(var) == type('a') :
			alpha = 'abcdefghijklmnopqrstuvwxyz'
			nums = '0123456789'
			
			if not '\n' in var :
				if var[0] in alpha :
					for i in range(1,len(var)) :
						if not var[i] in alpha and not var[i] in nums :
							print("Incorrect Variable name {}".format(var))
							return False
					return True
				if var[0] in nums :
					for i in range(1,len(var)) :
						if var[i] in alpha :
							print("Incorrect variable name {}".format(var))
							return False
					return True
			print("string given to check_varaible(str)  must be of single line")
			return self.error_code
	
	def Check_Indent(self,line,bias_indent=0) :
		if type(line) == type('a') :
			if '\n' in line :
				print("str given to Check_Indent func must be a single line string")
				return self.error_code
			count_indent = 0
			for char in line :
				if char == ' ' :
					count_indent += 1
				else :
					break
			if count_indent == len(self.indent) + bias_indent :
				return True
			print("incorrect number of indent {} provided at line {}".format(count_indent,line))
			return False
		print("line arg to Check_Indent() func must be a string ")
		return self.error_code
	
	def FetchLoopBody(self,lines,bias_indent=0) :
		if type(lines) == type('a') :
			if '\n' in lines :
				lines = lines.split('\n')
		if lines == [] :
			return
		loop_body = []
		pos = -1
		for i in range(len(lines)) :
			line = lines[i]
			nes_ind = None
			if self.Check_Syntax(line) :
				nes_loop,ind = self.FetchLoopBody(lines[i+1:],bias_indent+8)
			if self.Check_Indent(line,bias_indent) :
				loop_body.append(line)
			else :
				pos = i
		return loop_body,pos
		
	def HasNestedLoop(self,loop_body,bias_indent=4) :
		if loop_body :
			for i in range(len(loop_body)) :
				line = loop_body[i]
				if self.Check_Syntax(line) :
					return i
		return 0
	
	def ProcessLoop(self,lines,bias_indent=0) :
		if type(lines) == type('a') :
			if '\n' in lines :
				lines = lines.split('\n')
		if lines == [] :
			return
		loop_bodies = {}
		count = 0
		i = 0
		while i < len(lines) :
			line = lines[i]
			if self.Check_Syntax(line) :
				loop_body,_ = self.FetchLoopBody(lines[i+1:],bias_indent)
				nes_loop_body = []
				line_n = self.HasNestedLoop(loop_body,8+bias_indent)
				if line_n :
					nes_loop_body = self.ProcessLoop(loop_body[line_n:],bias_indent=8)
				loop_bodies[str(count)] = [loop_body,nes_loop_body]
				i = i + len(loop_body) 
			else :
				i = i + 1
		return loop_bodies
	
	def Check_Syntax(self,line) :
		if type(line) == type('a') :
			line = line.split()
		if len(line) == 7 :
			if line[0] == 'for' :
				#for i = a to b :
				i = line[1]
				equal_notation = line[2]
				a = line[3]
				to = line[4]
				b = line[5]
				indent = line[6]
				vars = [i,a,b]
				has_incorrect_syntax = False
				has_incorrect_var = False
				syntaxs = [equal_notation,to,indent]
				for i in range(len(self.for_loop_syntax)) :
					if not syntaxs[i] == self.for_loop_syntax[i] :
						print('incorrect syntax {}'.format(syntaxs[i]))
						has_incorrect_syntax = True
				if has_incorrect_syntax :
					return self.error_code
				for v in vars :
					if self.check_variable(v) :
						pass
					else :
						has_incorrect_var = True
				
				if has_incorrect_var :
					return self.error_code
				return True
				
			if len(line) == 5 :
				if line[0] == 'while' :
					#while a [<,>,<=,>=] b :
					a = line[1]
					op = line[2]
					b = line[3]
					ind = line[4]
					vars = [a,b]
					syntax = [op,ind]
					has_incorrect_var = False
					has_incorrect_syn = False
					for i in range(len(syntax)) :
						ops = self.while_loop_syntax[i]
						if i == 0 :
							ops = ops.split(',')
							if not syntax[i] in ops :
								print("incorrect syntax {}".format(syntax[i]))
								has_incorrect_syn = True
						
						if syntax[i] != ops :
							print("incorrect syntax {}".format(syntax[i]))
							has_incorrect_syn = True
					if has_incorrect_syn :
						return self.error_code
					for v in vars :
						if not self.check_variable(v) :
							has_incorrect_var = True
					if has_incorrect_var :
						return self.error_code
					return True
				return False

			
#LPU = LoopProcessingUnit()
lp = """
for i = 1 to n :
    a = a + 1
    while j < r :
        b = b + r
        for h = 1 to n :
            a = a + 1
    return
return a

"""
#print(LPU.ProcessLoop(lp))
			